#include <stdint.h>
#include <string.h>
#include "patcher.h"
#include <stdlib.h>

//***********************************************************************
//* Signature search and patch overlay
//*
//* ptype=0 - nop-patch
//* ptype=1 - br-patch
//***********************************************************************
uint32_t patch(struct defpatch fp, uint8_t* buf, uint32_t fsize, uint32_t ptype) {

// patch overlay - mov r0,#0
const char nop0[4]={0, 0, 0xa0, 0xe3};   
uint32_t i;
uint8_t c;

for(i=8;i<(fsize-60);i+=4) {
  if (memcmp(buf+i,fp.sig, fp.sigsize) == 0) {
    // found the signature - put a patch and leave
    switch (ptype) {
      case 0:
        memcpy(buf+i+fp.sigsize+fp.poffset,nop0,4);
        return i;

      case 1:
        c=*(buf+i+fp.sigsize+fp.poffset);
        c|=0xe0;
        *(buf+i+fp.sigsize+fp.poffset)=c;
        return i;

      default:
    exit(11);
    }
  }
}
// signature not found
return 0;
}

//**********************************************
// Patch descriptors
//**********************************************

const char sigburn_v7r22[]={
   0x12, 0x3B, 0xA0, 0xE3, 0x4A, 0x35, 0x4A, 0xE3, 0x00, 0x20, 0xA0, 0xE3,
   0x78, 0x20, 0xC3, 0xE5, 0x79, 0x20, 0xC3, 0xE5, 0x7A, 0x20, 0xC3, 0xE5, 
   0x7B, 0x20, 0xC3, 0xE5, 0x00, 0x00, 0xA0, 0xE3};

const char sigburn_v7r22_2[]={
   0x18, 0x30, 0x94, 0xE5, 0x10, 0x20, 0x94, 0xE5, 0x0D, 0x00, 0xA0, 0xE1,
   0x30, 0x40, 0x84, 0xE2, 0x14, 0x30, 0x8D, 0xE5, 0x10, 0x20, 0x8D, 0xE5};

const char sigburn_v7r11[]={
   0x00, 0x00, 0x50, 0xE3, 0x70, 0x80, 0xBD, 0x08, 0x00, 0x30, 0xA0, 0xE3, 
   0x4E, 0x26, 0x04, 0xE3, 0xE0, 0x3F, 0x44, 0xE3, 0x55, 0x22, 0x44, 0xE3, 
   0x4C, 0x01, 0x9F, 0xE5, 0x4E, 0xC6, 0x04, 0xE3, 0x48, 0x41, 0x9F, 0xE5, 
   0x02, 0x10, 0xA0, 0xE1, 0x4C, 0xC4, 0x44, 0xE3, 0x5C, 0x23, 0x83, 0xE5, 
   0x00, 0x00, 0x8F, 0xE0, 0x40, 0xC3, 0x83, 0xE5};

const char sigburn_v7r2[]={   
   0x00, 0x30, 0xA0, 0xE3, 0x9A, 0x3F, 0x07, 0xEE, 0xFF, 0x2F, 0x0F, 0xE3,
   0xE1, 0x2F, 0x44, 0xE3, 0x07, 0x30, 0x02, 0xE5, 0x9A, 0x3F, 0x07, 0xEE, 
   0x00, 0x40, 0xA0, 0xE3, 0xE0, 0x4F, 0x44, 0xE3, 0x4E, 0x36, 0x04, 0xE3, 
   0x4C, 0x34, 0x44, 0xE3, 0x30, 0x33, 0x84, 0xE5};

const char sigburn_v7r1[]={   
   0x3D, 0xE2, 0xE0, 0xE3, 0x00, 0xE0, 0x9E, 0xE5, 0x5C, 0xC3, 0x9F, 0xE5, 
   0x0C, 0x00, 0x5E, 0xE1, 0x00, 0x00, 0x00, 0x0A};
   
const char sigbad[]={0x04, 0x10, 0x8D, 0xE2, 0x04, 0x00, 0xA0, 0xE1};

struct defpatch patch_v7r22={sigburn_v7r22, sizeof(sigburn_v7r22), -37};
struct defpatch patch_v7r22_2={sigburn_v7r22_2, sizeof(sigburn_v7r22_2), 0};
struct defpatch patch_v7r11={sigburn_v7r11, sizeof(sigburn_v7r11), 4};   
struct defpatch patch_v7r2={sigburn_v7r2, sizeof(sigburn_v7r2), 16};   
struct defpatch patch_v7r1={sigburn_v7r1, sizeof(sigburn_v7r1), 0};   
struct defpatch patch_erasebad={sigbad, sizeof(sigbad), 0};   
   

//****************************************************
//* Patch procedures for different chipsets and tasks
//****************************************************

uint32_t pv7r2 (uint8_t* buf, uint32_t fsize) { return patch(patch_v7r2, buf, fsize,0); }
uint32_t pv7r11 (uint8_t* buf, uint32_t fsize) { return patch(patch_v7r11, buf, fsize,0); }
uint32_t pv7r1 (uint8_t* buf, uint32_t fsize) { return patch(patch_v7r1, buf, fsize,0); }
uint32_t pv7r22 (uint8_t* buf, uint32_t fsize) { return patch(patch_v7r22, buf, fsize,1); }
uint32_t pv7r22_2 (uint8_t* buf, uint32_t fsize) { return patch(patch_v7r22_2, buf, fsize,0); }
uint32_t perasebad (uint8_t* buf, uint32_t fsize) { return patch(patch_erasebad, buf, fsize,0); }

